<?php

namespace Litecms\Plans\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Plans\Http\Requests\PlansRequest;
use Litecms\Plans\Interfaces\PlansRepositoryInterface;
use Litecms\Plans\Models\Plans;

/**
 * Resource controller class for plans.
 */
class PlansResourceController extends BaseController
{

    /**
     * Initialize plans resource controller.
     *
     * @param type PlansRepositoryInterface $plans
     *
     * @return null
     */
    public function __construct(PlansRepositoryInterface $plans)
    {
        parent::__construct();
        $this->repository = $plans;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Plans\Repositories\Criteria\PlansResourceCriteria::class);
    }

    /**
     * Display a list of plans.
     *
     * @return Response
     */
    public function index(PlansRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Plans\Repositories\Presenter\PlansPresenter::class)
                ->$function();
        }

        $plans = $this->repository->paginate();

        return $this->response->title(trans('plans::plans.names'))
            ->view('plans::plans.index', true)
            ->data(compact('plans'))
            ->output();
    }

    /**
     * Display plans.
     *
     * @param Request $request
     * @param Model   $plans
     *
     * @return Response
     */
    public function show(PlansRequest $request, Plans $plans)
    {

        if ($plans->exists) {
            $view = 'plans::plans.show';
        } else {
            $view = 'plans::plans.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('plans::plans.name'))
            ->data(compact('plans'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new plans.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PlansRequest $request)
    {

        $plans = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('plans::plans.name')) 
            ->view('plans::plans.create', true) 
            ->data(compact('plans'))
            ->output();
    }

    /**
     * Create new plans.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PlansRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $plans                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('plans::plans.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plans/plans/' . $plans->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/plans/plans'))
                ->redirect();
        }

    }

    /**
     * Show plans for editing.
     *
     * @param Request $request
     * @param Model   $plans
     *
     * @return Response
     */
    public function edit(PlansRequest $request, Plans $plans)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('plans::plans.name'))
            ->view('plans::plans.edit', true)
            ->data(compact('plans'))
            ->output();
    }

    /**
     * Update the plans.
     *
     * @param Request $request
     * @param Model   $plans
     *
     * @return Response
     */
    public function update(PlansRequest $request, Plans $plans)
    {
        try {
            $attributes = $request->all();

            $plans->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('plans::plans.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plans/plans/' . $plans->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plans/plans/' . $plans->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the plans.
     *
     * @param Model   $plans
     *
     * @return Response
     */
    public function destroy(PlansRequest $request, Plans $plans)
    {
        try {

            $plans->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plans::plans.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('plans/plans/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plans/plans/' . $plans->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple plans.
     *
     * @param Model   $plans
     *
     * @return Response
     */
    public function delete(PlansRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plans::plans.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('plans/plans'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plans/plans'))
                ->redirect();
        }

    }

    /**
     * Restore deleted plans.
     *
     * @param Model   $plans
     *
     * @return Response
     */
    public function restore(PlansRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('plans::plans.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/plans/plans'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plans/plans/'))
                ->redirect();
        }

    }

}
